'use strict';

module.exports = function (Promise) {
  // eslint-disable-line no-shadow
  var prototype = Promise.prototype;

  // How many milliseconds at most we can wait the promise fulfilled
  prototype.timeout = function timeout(ms, message) {
    if (ms > 0) {
      return Promise.race([this, new Promise(function (resolve, reject) {
        setTimeout(function () {
          var err = new Error(message || 'timeout');
          err.timeout = true;
          reject(err);
        }, ms);
      })]);
    }
    return this;
  };

  // How many milliseconds at least the promise would be treated as fulfilled
  prototype.delay = function delay(ms) {
    return Promise.all([this, Promise.delay(ms)]).then(function (result) {
      return result[0];
    });
  };

  prototype.finally = function finalli(callback) {
    /* eslint-disable promise/catch-or-return */
    return this.then(function (result) {
      try {
        return Promise.resolve(callback()).catch(function (e) {
          Promise.reject(e); // trigger global unhandledRejection event
        }).then(function () {
          return result;
        });
      } catch (e) {
        Promise.reject(e); // trigger global unhandledRejection event
        return result;
      }
    }, function (err) {
      try {
        return Promise.resolve(callback()).catch(function (e) {
          Promise.reject(e); // trigger global unhandledRejection event
        }).then(function () {
          return Promise.reject(err);
        });
      } catch (e) {
        Promise.reject(e); // trigger global unhandledRejection event
        return Promise.reject(err);
      }
      /* eslint-enable promise/catch-or-return */
    });
  };

  prototype.asCallback = function asCallback(callback) {
    return this.then(function (ret) {
      return callback(null, ret);
    }, function (err) {
      return callback(err);
    });
  };
  prototype.nodeify = prototype.asCallback;
  prototype.toCallback = prototype.asCallback;
};