'use strict';

module.exports = function (Promise) {
  // eslint-disable-line no-shadow
  Promise.delay = function (ms) {
    return new Promise(function (resolve) {
      return setTimeout(resolve, ms);
    });
  };

  function promisify(funcWithCallback) {
    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        context = _ref.context,
        _ref$multiArgs = _ref.multiArgs,
        multiArgs = _ref$multiArgs === undefined ? false : _ref$multiArgs;

    if (typeof funcWithCallback !== 'function') throw new Error('Only allow function to be promisified');
    return function promisifiedFunction() {
      var _this = this;

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      // eslint-disable-line func-names
      return new Promise(function (resolve, reject) {
        funcWithCallback.call.apply(funcWithCallback, [context || _this].concat(args, [function (err) {
          for (var _len2 = arguments.length, rets = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
            rets[_key2 - 1] = arguments[_key2];
          }

          if (err) return reject(err);else {
            if (multiArgs) return resolve(rets);
            return resolve(rets[0]);
          }
        }]));
      });
    };
  }

  // Promise.promisify(fs.readFile);
  // Promise.promisify([fs.readFile, fs.writeFile])
  Promise.promisify = function (input, options) {
    if (Array.isArray(input)) {
      return input.map(function (func) {
        return promisify(input, options);
      });
    } else {
      return promisify(input, options);
    }
  };
  Promise.denodeify = Promise.promisify;

  Promise.promisifyAll = function (obj) {
    var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref2$suffix = _ref2.suffix,
        suffix = _ref2$suffix === undefined ? 'Async' : _ref2$suffix,
        _ref2$multiArgs = _ref2.multiArgs,
        multiArgs = _ref2$multiArgs === undefined ? false : _ref2$multiArgs,
        _ref2$filter = _ref2.filter,
        filter = _ref2$filter === undefined ? defaultFilter : _ref2$filter;

    if (obj.__PromiseAdditionPromisified__) return obj;
    obj.__PromiseAdditionPromisified__ = true;

    for (var key in obj) {
      var value = obj[key];
      if (typeof value !== 'function') continue;
      if (!filter(value, key, obj)) continue;
      obj[key + suffix] = promisify(value, { multiArgs: multiArgs });
    }
    return obj;
  };
  Promise.denodeifyAll = Promise.promisifyAll;

  // Promise.fromCallback((cb)=>cb(null, result));
  Promise.fromCallback = function fromCallback(func) {
    var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref3$multiArgs = _ref3.multiArgs,
        multiArgs = _ref3$multiArgs === undefined ? false : _ref3$multiArgs;

    return new Promise(function (resolve, reject) {
      func(function (err) {
        for (var _len3 = arguments.length, rets = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
          rets[_key3 - 1] = arguments[_key3];
        }

        if (err) return reject(err);

        if (multiArgs) return resolve(rets);
        return resolve(rets[0]);
      });
    });
  };
  Promise.fromNode = Promise.fromCallback;

  // run the promises concurrently and return the result in a array with original order
  Promise.map = function (input, iterator) {
    var _ref4 = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
        _ref4$concurrency = _ref4.concurrency,
        concurrency = _ref4$concurrency === undefined ? Infinity : _ref4$concurrency;

    return Promise.resolve(input).then(function (arr) {
      if (concurrency === Infinity || !(concurrency > 0)) return Promise.all(arr.map(iterator));

      var len = arr.length;
      var result = new Array(len);

      var next = 0; // next one that will be run
      var error = null; // if an error occured, all remain ones will not be run

      function run(index) {
        if (index >= len || error) return Promise.resolve();
        next += 1;

        try {
          return Promise.resolve(iterator(arr[index], index)).then(function (ret) {
            result[index] = ret;
            return run(next);
          }, function (err) {
            error = err;
            return Promise.reject(err);
          });
        } catch (e) {
          return Promise.reject(e);
        }
      }

      return Promise.all(new Array(concurrency).fill().map(function (val, index) {
        return run(index);
      })).then(function () {
        return result;
      }, function (err) {
        error = null;
        next = null;
        return Promise.reject(err);
      });
    });
  };

  // run the promises one by one and return the result in a array with original order
  Promise.each = function (input, iterator) {
    return Promise.map(input, iterator, { concurrency: 1 });
  };

  // run the promises one by one and reduce the array result to a value
  Promise.reduce = function (input, iterator, initialValue) {
    return Promise.resolve(input).then(function (arr) {
      return arr.reduce(function (prev, value, index) {
        return prev.then(function (prevRet) {
          return iterator(prevRet, value, index);
        });
      }, Promise.resolve(initialValue));
    });
  };
};

function defaultFilter(value, key, obj) {
  return true;
}